import { Injectable, EventEmitter, ChangeDetectorRef, ApplicationRef, NgZone } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Observable, timer } from 'rxjs';
import 'rxjs/add/operator/share';
import { BehaviorSubject } from 'rxjs';
import { GlobalService } from '../global/global.service';
import { environment } from '../../../../src/environments/environment';
import { ActivatedRoute, NavigationCancel, NavigationEnd, NavigationStart, Router } from '@angular/router';
import { FlowCookieService } from 'flow-concept-shared/services/cookie/cookie.service';
import { catchError } from 'rxjs/operators';
declare var window: any;

@Injectable({
    providedIn: 'root'
  })
export class FlowHttpClient {

    public loading: number = 0;
    public pageLoadingComplete: EventEmitter<any> = new EventEmitter(false);
    public loadingSuccess: EventEmitter<any> = new EventEmitter(false);
    public pageLoadingStart: EventEmitter<any> = new EventEmitter(false);
    public loadingFailure: EventEmitter<any> = new EventEmitter(false);

    public _pageLoading: number = 0;
    private _spinLoading: number = 0;
    public set pageLoading(value) {
        let changed = value!=this.loading;
        this.loading = value;
        if ((this.recordPageLoading && value > this._pageLoading) || value < this._pageLoading){
            if (this._pageLoading == 0)
                this.pageLoadingStart.emit();
                
            this._pageLoading = value;
        }
        if (value <= 0) {
            this.recordPageLoading = false;
            this._pageLoading = 0;
            this.pageLoadingComplete.emit();
        }

        if (this.loading<=0 && changed) {
            this.loadingSuccess.emit();
            timer(500).subscribe(res => window.prerenderReady = true);

        }
    }
    public get pageLoading(){   
        if (this.globalService.browser.name=='Safari' && this.globalService.browser.version && parseInt(this.globalService.browser.version.substring(0,1)) < 9)
            return 0;
        return this._pageLoading;
    }

    public recordPageLoading: boolean = true;

    public onError: EventEmitter<any> = new EventEmitter(false);


    private _token: string;
    public get token(): string {
        if (!this._token)
            this._token = this.cookieService.get('auth_token');

        var guidRegex = /^(?:\{{0,1}(?:[0-9a-fA-F]){8}-(?:[0-9a-fA-F]){4}-(?:[0-9a-fA-F]){4}-(?:[0-9a-fA-F]){4}-(?:[0-9a-fA-F]){12}\}{0,1})$/;
        if (!guidRegex.test(this._token)){
            this._token = null;
            this.cookieService.remove('auth_token');
        }
        return this._token;
    }

    public set token(value) { 
        this.cookieService.remove('auth_token');
        this._token = value;
        if (value)
            this.cookieService.put('auth_token', value);
    }

    constructor(public http: HttpClient, public cookieService: FlowCookieService, public globalService: GlobalService, private ref: ApplicationRef, public zone: NgZone,
        public route: ActivatedRoute,
        public router: Router)
    {
        this.route.queryParams.subscribe(r=> {
            if (r['auth_token']) {
                this.token = r['auth_token'];
                let obj = Object.assign({}, r);
                obj['auth_token'] = null;
                this.router.navigate([], {relativeTo: route, queryParams: obj, queryParamsHandling: 'merge'});
            }
        });

        this.router.events.subscribe(event => {
            if (event instanceof NavigationStart) {
              this.recordPageLoading = true;
              this.pageLoading++;
              jQuery("html, body").animate({ scrollTop: 0 }, 50);
            }
            if (event instanceof NavigationEnd || event instanceof NavigationCancel) {
              timer(100).subscribe(r=> this.pageLoading--);
            }
          });
    
    }

    get(endpoint: string, url: string, options?: any, showLoader=true):Observable<any> {

        if (showLoader)
            this.pageLoading++;
        options = this.createAuthHeader(options);
        let retValue = this.http.get(endpoint + url, options)
        .pipe(catchError((err, caught) => {
            this.recordPageLoading = false;
            this.pageLoading = 0;
            this.onError.emit(err);
            this.globalService.onError.emit();
            throw err;
        }))
            .share();
        retValue
            .subscribe(res => { 
                if (showLoader)
                    this.pageLoading--; 
            });
        return retValue;
    }

    delete(endpoint: string, url: string, options?: any, showLoader = false): Observable<any> {
        this.pageLoading++;

        options = this.createAuthHeader(options);
        let retValue = this.http.delete(endpoint + url, options)
        .pipe(catchError((err, caught) => {
            this.recordPageLoading = false;
            this.pageLoading = 0;
            this.onError.emit(err);
            this.globalService.onError.emit();
            throw err;
        })).share();
        retValue
            .subscribe(res => { 

                    this.pageLoading--; 
            });
        return retValue;
    }

    post(endpoint: string, url: string, data: any, options?: any, showLoader = true): Observable<any> {
        
        this.pageLoading++;
        options = this.createAuthHeader(options);
        if (data !== null && !options.headers.get('Content-Type'))
            options.headers = options.headers.set('Content-Type', 'application/json');
        let retValue = this.http.post(endpoint + url, data, options)
        .pipe(catchError((err, caught) => {
            this.recordPageLoading = false;
            this.pageLoading = 0;
            this.onError.emit(err);
            this.globalService.onError.emit();
            throw err;
        })).share();
        retValue
            
            .subscribe(res => { 
                
                    this.pageLoading--; 
            });
            return retValue;
    }

    put(endpoint: string, url: string, data: any, options?: any, showLoader = false): Observable<any> {
        this.pageLoading++;
 
        options = this.createAuthHeader(options);
        if (data !== null && !options.headers.get('Content-Type'))
        options.headers = options.headers.set('Content-Type', 'application/json');
        let retValue = this.http.put(endpoint + url, data, options)            
        .pipe(catchError((err, caught) => {
            this.recordPageLoading = false;
            this.pageLoading = 0;
            this.onError.emit(err);
            this.globalService.onError.emit();
            throw err;
        }))
        .share();
        retValue
        .subscribe(res => { 
                
            this.pageLoading--; 
        });
        return retValue;
    }

    public createAuthHeader(options: any) {
        if (!options)
            options = {};
        if (!options.headers) {
            options.headers = new HttpHeaders({ 
                'Accept': 'application/json', 
                'flow-language-code': this.globalService._currentLanguage,
                'flow-edit-language-code': this.globalService._currentEditLanguage,
            });
        } else {
            if (!options.headers.get('Accept'))
            options.headers = options.headers.set('Accept', 'application/json');
            if (!options.headers.get('flow-language-code') && !options.excludeExtraHeaders)
            options.headers = options.headers.set('flow-language-code', this.globalService._currentLanguage);
        }

        if (this.token && !options.excludeExtraHeaders) {
            options.headers = options.headers.set('Authorization', 'Bearer ' + this.token);
        }

        if(environment.forceOrigin && !options.excludeExtraHeaders) {
            options.headers = options.headers.set('flow-origin', environment.forceOrigin);
        }
        return options;
    }

    public serializeToQueryString(obj, prefix = null) {
        var str = [], p;
        for (p in obj) {
            if (obj.hasOwnProperty(p) && obj[p]!=null) {
                var k = prefix && !isNaN(p) ? prefix + "[" + p + "]" : prefix && isNaN(p) ? prefix + "." + p : p, v = obj[p];
                str.push((v !== null && typeof v === "object") ?
                    this.serializeToQueryString(v, k) :
                    encodeURIComponent(k) + "=" + encodeURIComponent(v));
            }
        }
        return str.join("&");
    }

    public serializeToQueryString2(obj) {
        var str = [], p;
        for (p in obj) {
            if (obj.hasOwnProperty(p) && obj[p]!=null) {
                var k = !isNaN(p) ? "[" + p + "]" : isNaN(p) ? p : p, v = obj[p];
                str.push((v !== null && typeof v === "object") ?
                    this.serializeToQueryString(v, k) :
                    encodeURIComponent(this.jsUcfirst(k)) + "=" + encodeURIComponent(v));
            }
        }
        return str.join("&");
    }

    jsUcfirst(string) 
    {
        return string.charAt(0).toUpperCase() + string.slice(1);
    }
}
